# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field

from . import core_v1
from . import meta_v1
from . import resource


@dataclass
class CSIDriver(DictMixin):
    """CSIDriver captures information about a Container Storage Interface (CSI)
      volume driver deployed on the cluster. Kubernetes attach detach controller
      uses this object to determine whether attach is required. Kubelet uses this
      object to determine whether pod information needs to be passed on mount.
      CSIDriver objects are non-namespaced.

      **parameters**

      * **spec** ``CSIDriverSpec`` - Specification of the CSI Driver.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata. metadata.Name indicates the name of the CSI driver
        that this object refers to; it MUST be the same name returned by the CSI
        GetPluginName() call for that driver. The driver name must be 63 characters or
        less, beginning and ending with an alphanumeric character ([a-z0-9A-Z]) with
        dashes (-), dots (.), and alphanumerics between. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    spec: 'CSIDriverSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None


@dataclass
class CSIDriverList(DictMixin):
    """CSIDriverList is a collection of CSIDriver objects.

      **parameters**

      * **items** ``List[CSIDriver]`` - items is the list of CSIDriver
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[CSIDriver]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class CSIDriverSpec(DictMixin):
    """CSIDriverSpec is the specification of a CSIDriver.

      **parameters**

      * **attachRequired** ``Optional[bool]`` - attachRequired indicates this CSI volume driver requires an attach operation
        (because it implements the CSI ControllerPublishVolume() method), and that the
        Kubernetes attach detach controller should call the attach volume interface
        which checks the volumeattachment status and waits until the volume is
        attached before proceeding to mounting. The CSI external-attacher coordinates
        with CSI volume driver and updates the volumeattachment status when the attach
        operation is complete. If the CSIDriverRegistry feature gate is enabled and
        the value is specified to false, the attach operation will be skipped.
        Otherwise the attach operation will be called.
        This field is immutable.
      * **fsGroupPolicy** ``Optional[str]`` - Defines if the underlying volume supports changing ownership and permission of
        the volume before being mounted. Refer to the specific FSGroupPolicy values
        for additional details.
        This field is immutable.
        Defaults to ReadWriteOnceWithFSType, which will examine each volume to
        determine if Kubernetes should modify ownership and permissions of the volume.
        With the default policy the defined fsGroup will only be applied if a fstype
        is defined and the volume's access mode contains ReadWriteOnce.
      * **podInfoOnMount** ``Optional[bool]`` - If set to true, podInfoOnMount indicates this CSI volume driver requires
        additional pod information (like podName, podUID, etc.) during mount
        operations. If set to false, pod information will not be passed on mount.
        Default is false. The CSI driver specifies podInfoOnMount as part of driver
        deployment. If true, Kubelet will pass pod information as VolumeContext in the
        CSI NodePublishVolume() calls. The CSI driver is responsible for parsing and
        validating the information passed in as VolumeContext. The following
        VolumeConext will be passed if podInfoOnMount is set to true. This list might
        grow, but the prefix will be used. "csi.storage.k8s.io/pod.name": pod.Name
        "csi.storage.k8s.io/pod.namespace": pod.Namespace
        "csi.storage.k8s.io/pod.uid": string(pod.UID) "csi.storage.k8s.io/ephemeral":
        "true" if the volume is an ephemeral inline volume
                                        defined by a CSIVolumeSource, otherwise
        "false"
        "csi.storage.k8s.io/ephemeral" is a new feature in Kubernetes 1.16. It is only
        required for drivers which support both the "Persistent" and "Ephemeral"
        VolumeLifecycleMode. Other drivers can leave pod info disabled and/or ignore
        this field. As Kubernetes 1.15 doesn't support this field, drivers can only
        support one mode when deployed on such a cluster and the deployment determines
        which mode that is, for example via a command line parameter of the driver.
        This field is immutable.
      * **requiresRepublish** ``Optional[bool]`` - RequiresRepublish indicates the CSI driver wants `NodePublishVolume` being
        periodically called to reflect any possible change in the mounted volume. This
        field defaults to false.
        Note: After a successful initial NodePublishVolume call, subsequent calls to
        NodePublishVolume should only update the contents of the volume. New mount
        points will not be seen by a running container.
      * **seLinuxMount** ``Optional[bool]`` - SELinuxMount specifies if the CSI driver supports "-o context" mount option.
        When "true", the CSI driver must ensure that all volumes provided by this CSI
        driver can be mounted separately with different `-o context` options. This is
        typical for storage backends that provide volumes as filesystems on block
        devices or as independent shared volumes. Kubernetes will call NodeStage /
        NodePublish with "-o context=xyz" mount option when mounting a
        ReadWriteOncePod volume used in Pod that has explicitly set SELinux context.
        In the future, it may be expanded to other volume AccessModes. In any case,
        Kubernetes will ensure that the volume is mounted only with a single SELinux
        context.
        When "false", Kubernetes won't pass any special SELinux mount options to the
        driver. This is typical for volumes that represent subdirectories of a bigger
        shared filesystem.
        Default is "false".
      * **storageCapacity** ``Optional[bool]`` - If set to true, storageCapacity indicates that the CSI volume driver wants pod
        scheduling to consider the storage capacity that the driver deployment will
        report by creating CSIStorageCapacity objects with capacity information.
        The check can be enabled immediately when deploying a driver. In that case,
        provisioning new volumes with late binding will pause until the driver
        deployment has published some suitable CSIStorageCapacity object.
        Alternatively, the driver can be deployed with the field unset or false and it
        can be flipped later when storage capacity information has been published.
        This field was immutable in Kubernetes <= 1.22 and now is mutable.
      * **tokenRequests** ``Optional[List[TokenRequest]]`` - TokenRequests indicates the CSI driver needs pods' service account tokens it
        is mounting volume for to do necessary authentication. Kubelet will pass the
        tokens in VolumeContext in the CSI NodePublishVolume calls. The CSI driver
        should parse and validate the following VolumeContext:
        "csi.storage.k8s.io/serviceAccount.tokens": {
          "<audience>": {
            "token": <token>,
            "expirationTimestamp": <expiration timestamp in RFC3339>,
          },
          ...
        }
        Note: Audience in each TokenRequest should be different and at most one token
        is empty string. To receive a new token after expiry, RequiresRepublish can be
        used to trigger NodePublishVolume periodically.
      * **volumeLifecycleModes** ``Optional[List[str]]`` - volumeLifecycleModes defines what kind of volumes this CSI volume driver
        supports. The default if the list is empty is "Persistent", which is the usage
        defined by the CSI specification and implemented in Kubernetes via the usual
        PV/PVC mechanism. The other mode is "Ephemeral". In this mode, volumes are
        defined inline inside the pod spec with CSIVolumeSource and their lifecycle is
        tied to the lifecycle of that pod. A driver has to be aware of this because it
        is only going to get a NodePublishVolume call for such a volume. For more
        information about implementing this mode, see
        https://kubernetes-csi.github.io/docs/ephemeral-local-volumes.html A driver
        can support one or more of these modes and more modes may be added in the
        future. This field is beta.
        This field is immutable.
    """
    attachRequired: 'Optional[bool]' = None
    fsGroupPolicy: 'Optional[str]' = None
    podInfoOnMount: 'Optional[bool]' = None
    requiresRepublish: 'Optional[bool]' = None
    seLinuxMount: 'Optional[bool]' = None
    storageCapacity: 'Optional[bool]' = None
    tokenRequests: 'Optional[List[TokenRequest]]' = None
    volumeLifecycleModes: 'Optional[List[str]]' = None


@dataclass
class CSINode(DictMixin):
    """CSINode holds information about all CSI drivers installed on a node. CSI
      drivers do not need to create the CSINode object directly. As long as they use
      the node-driver-registrar sidecar container, the kubelet will automatically
      populate the CSINode object for the CSI driver as part of kubelet plugin
      registration. CSINode has the same name as a node. If the object is missing,
      it means either there are no CSI Drivers available on the node, or the Kubelet
      version is low enough that it doesn't create this object. CSINode has an
      OwnerReference that points to the corresponding node object.

      **parameters**

      * **spec** ``CSINodeSpec`` - spec is the specification of CSINode
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - metadata.name must be the Kubernetes node name.
    """
    spec: 'CSINodeSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None


@dataclass
class CSINodeDriver(DictMixin):
    """CSINodeDriver holds information about the specification of one CSI driver
      installed on a node

      **parameters**

      * **name** ``str`` - This is the name of the CSI driver that this object refers to. This MUST be
        the same name returned by the CSI GetPluginName() call for that driver.
      * **nodeID** ``str`` - nodeID of the node from the driver point of view. This field enables
        Kubernetes to communicate with storage systems that do not share the same
        nomenclature for nodes. For example, Kubernetes may refer to a given node as
        "node1", but the storage system may refer to the same node as "nodeA". When
        Kubernetes issues a command to the storage system to attach a volume to a
        specific node, it can use this field to refer to the node name using the ID
        that the storage system will understand, e.g. "nodeA" instead of "node1". This
        field is required.
      * **allocatable** ``Optional[VolumeNodeResources]`` - allocatable represents the volume resources of a node that are available for
        scheduling. This field is beta.
      * **topologyKeys** ``Optional[List[str]]`` - topologyKeys is the list of keys supported by the driver. When a driver is
        initialized on a cluster, it provides a set of topology keys that it
        understands (e.g. "company.com/zone", "company.com/region"). When a driver is
        initialized on a node, it provides the same topology keys along with values.
        Kubelet will expose these topology keys as labels on its own node object. When
        Kubernetes does topology aware provisioning, it can use this list to determine
        which labels it should retrieve from the node object and pass back to the
        driver. It is possible for different nodes to use different topology keys.
        This can be empty if driver does not support topology.
    """
    name: 'str'
    nodeID: 'str'
    allocatable: 'Optional[VolumeNodeResources]' = None
    topologyKeys: 'Optional[List[str]]' = None


@dataclass
class CSINodeList(DictMixin):
    """CSINodeList is a collection of CSINode objects.

      **parameters**

      * **items** ``List[CSINode]`` - items is the list of CSINode
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[CSINode]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class CSINodeSpec(DictMixin):
    """CSINodeSpec holds information about the specification of all CSI drivers
      installed on a node

      **parameters**

      * **drivers** ``List[CSINodeDriver]`` - drivers is a list of information of all CSI Drivers existing on a node. If all
        drivers in the list are uninstalled, this can become empty.
    """
    drivers: 'List[CSINodeDriver]'


@dataclass
class CSIStorageCapacity(DictMixin):
    """CSIStorageCapacity stores the result of one CSI GetCapacity call. For a given
      StorageClass, this describes the available capacity in a particular topology
      segment.  This can be used when considering where to instantiate new
      PersistentVolumes.
      
      For example this can express things like: - StorageClass "standard" has "1234
      GiB" available in "topology.kubernetes.io/zone=us-east1" - StorageClass
      "localssd" has "10 GiB" available in "kubernetes.io/hostname=knode-abc123"
      
      The following three cases all imply that no capacity is available for a
      certain combination: - no object exists with suitable topology and storage
      class name - such an object exists, but the capacity is unset - such an object
      exists, but the capacity is zero
      
      The producer of these objects can decide which approach is more suitable.
      
      They are consumed by the kube-scheduler when a CSI driver opts into
      capacity-aware scheduling with CSIDriverSpec.StorageCapacity. The scheduler
      compares the MaximumVolumeSize against the requested size of pending volumes
      to filter out unsuitable nodes. If MaximumVolumeSize is unset, it falls back
      to a comparison against the less precise Capacity. If that is also unset, the
      scheduler assumes that capacity is insufficient and tries some other node.

      **parameters**

      * **storageClassName** ``str`` - The name of the StorageClass that the reported capacity applies to. It must
        meet the same requirements as the name of a StorageClass object (non-empty,
        DNS subdomain). If that object no longer exists, the CSIStorageCapacity object
        is obsolete and should be removed by its creator. This field is immutable.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **capacity** ``Optional[resource.Quantity]`` - Capacity is the value reported by the CSI driver in its GetCapacityResponse
        for a GetCapacityRequest with topology and parameters that match the previous
        fields.
        The semantic is currently (CSI spec 1.2) defined as: The available capacity,
        in bytes, of the storage that can be used to provision volumes. If not set,
        that information is currently unavailable.
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **maximumVolumeSize** ``Optional[resource.Quantity]`` - MaximumVolumeSize is the value reported by the CSI driver in its
        GetCapacityResponse for a GetCapacityRequest with topology and parameters that
        match the previous fields.
        This is defined since CSI spec 1.4.0 as the largest size that may be used in a
        CreateVolumeRequest.capacity_range.required_bytes field to create a volume
        with the same parameters as those in GetCapacityRequest. The corresponding
        value in the Kubernetes API is ResourceRequirements.Requests in a volume
        claim.
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. The name has no particular meaning. It must be be
        a DNS subdomain (dots allowed, 253 characters). To ensure that there are no
        conflicts with other CSI drivers on the cluster, the recommendation is to use
        csisc-<uuid>, a generated name, or a reverse-domain name which ends with the
        unique CSI driver name.
        Objects are namespaced.
        More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **nodeTopology** ``Optional[meta_v1.LabelSelector]`` - NodeTopology defines which nodes have access to the storage for which capacity
        was reported. If not set, the storage is not accessible from any node in the
        cluster. If empty, the storage is accessible from all nodes. This field is
        immutable.
    """
    storageClassName: 'str'
    apiVersion: 'Optional[str]' = None
    capacity: 'Optional[resource.Quantity]' = None
    kind: 'Optional[str]' = None
    maximumVolumeSize: 'Optional[resource.Quantity]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    nodeTopology: 'Optional[meta_v1.LabelSelector]' = None


@dataclass
class CSIStorageCapacityList(DictMixin):
    """CSIStorageCapacityList is a collection of CSIStorageCapacity objects.

      **parameters**

      * **items** ``List[CSIStorageCapacity]`` - Items is the list of CSIStorageCapacity objects.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[CSIStorageCapacity]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class StorageClass(DictMixin):
    """StorageClass describes the parameters for a class of storage for which
      PersistentVolumes can be dynamically provisioned.
      
      StorageClasses are non-namespaced; the name of the storage class according to
      etcd is in ObjectMeta.Name.

      **parameters**

      * **provisioner** ``str`` - Provisioner indicates the type of the provisioner.
      * **allowVolumeExpansion** ``Optional[bool]`` - AllowVolumeExpansion shows whether the storage class allow volume expand
      * **allowedTopologies** ``Optional[List[core_v1.TopologySelectorTerm]]`` - Restrict the node topologies where volumes can be dynamically provisioned.
        Each volume plugin defines its own supported topology specifications. An empty
        TopologySelectorTerm list means there is no topology restriction. This field
        is only honored by servers that enable the VolumeScheduling feature.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **mountOptions** ``Optional[List[str]]`` - Dynamically provisioned PersistentVolumes of this storage class are created
        with these mountOptions, e.g. ["ro", "soft"]. Not validated - mount of the PVs
        will simply fail if one is invalid.
      * **parameters** ``Optional[dict]`` - Parameters holds the parameters for the provisioner that should create volumes
        of this storage class.
      * **reclaimPolicy** ``Optional[str]`` - Dynamically provisioned PersistentVolumes of this storage class are created
        with this reclaimPolicy. Defaults to Delete.
      * **volumeBindingMode** ``Optional[str]`` - VolumeBindingMode indicates how PersistentVolumeClaims should be provisioned
        and bound.  When unset, VolumeBindingImmediate is used. This field is only
        honored by servers that enable the VolumeScheduling feature.
    """
    provisioner: 'str'
    allowVolumeExpansion: 'Optional[bool]' = None
    allowedTopologies: 'Optional[List[core_v1.TopologySelectorTerm]]' = None
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    mountOptions: 'Optional[List[str]]' = None
    parameters: 'Optional[dict]' = None
    reclaimPolicy: 'Optional[str]' = None
    volumeBindingMode: 'Optional[str]' = None


@dataclass
class StorageClassList(DictMixin):
    """StorageClassList is a collection of storage classes.

      **parameters**

      * **items** ``List[StorageClass]`` - Items is the list of StorageClasses
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[StorageClass]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class TokenRequest(DictMixin):
    """TokenRequest contains parameters of a service account token.

      **parameters**

      * **audience** ``str`` - Audience is the intended audience of the token in "TokenRequestSpec". It will
        default to the audiences of kube apiserver.
      * **expirationSeconds** ``Optional[int]`` - ExpirationSeconds is the duration of validity of the token in
        "TokenRequestSpec". It has the same default value of "ExpirationSeconds" in
        "TokenRequestSpec".
    """
    audience: 'str'
    expirationSeconds: 'Optional[int]' = None


@dataclass
class VolumeAttachment(DictMixin):
    """VolumeAttachment captures the intent to attach or detach the specified volume
      to/from the specified node.
      
      VolumeAttachment objects are non-namespaced.

      **parameters**

      * **spec** ``VolumeAttachmentSpec`` - Specification of the desired attach/detach volume behavior. Populated by the
        Kubernetes system.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **status** ``Optional[VolumeAttachmentStatus]`` - Status of the VolumeAttachment request. Populated by the entity completing the
        attach or detach operation, i.e. the external-attacher.
    """
    spec: 'VolumeAttachmentSpec'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    status: 'Optional[VolumeAttachmentStatus]' = None


@dataclass
class VolumeAttachmentList(DictMixin):
    """VolumeAttachmentList is a collection of VolumeAttachment objects.

      **parameters**

      * **items** ``List[VolumeAttachment]`` - Items is the list of VolumeAttachments
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[VolumeAttachment]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class VolumeAttachmentSource(DictMixin):
    """VolumeAttachmentSource represents a volume that should be attached. Right now
      only PersistenVolumes can be attached via external attacher, in future we may
      allow also inline volumes in pods. Exactly one member can be set.

      **parameters**

      * **inlineVolumeSpec** ``Optional[core_v1.PersistentVolumeSpec]`` - inlineVolumeSpec contains all the information necessary to attach a persistent
        volume defined by a pod's inline VolumeSource. This field is populated only
        for the CSIMigration feature. It contains translated fields from a pod's
        inline VolumeSource to a PersistentVolumeSpec. This field is beta-level and is
        only honored by servers that enabled the CSIMigration feature.
      * **persistentVolumeName** ``Optional[str]`` - Name of the persistent volume to attach.
    """
    inlineVolumeSpec: 'Optional[core_v1.PersistentVolumeSpec]' = None
    persistentVolumeName: 'Optional[str]' = None


@dataclass
class VolumeAttachmentSpec(DictMixin):
    """VolumeAttachmentSpec is the specification of a VolumeAttachment request.

      **parameters**

      * **attacher** ``str`` - Attacher indicates the name of the volume driver that MUST handle this
        request. This is the name returned by GetPluginName().
      * **nodeName** ``str`` - The node that the volume should be attached to.
      * **source** ``VolumeAttachmentSource`` - Source represents the volume that should be attached.
    """
    attacher: 'str'
    nodeName: 'str'
    source: 'VolumeAttachmentSource'


@dataclass
class VolumeAttachmentStatus(DictMixin):
    """VolumeAttachmentStatus is the status of a VolumeAttachment request.

      **parameters**

      * **attached** ``bool`` - Indicates the volume is successfully attached. This field must only be set by
        the entity completing the attach operation, i.e. the external-attacher.
      * **attachError** ``Optional[VolumeError]`` - The last error encountered during attach operation, if any. This field must
        only be set by the entity completing the attach operation, i.e. the
        external-attacher.
      * **attachmentMetadata** ``Optional[dict]`` - Upon successful attach, this field is populated with any information returned
        by the attach operation that must be passed into subsequent WaitForAttach or
        Mount calls. This field must only be set by the entity completing the attach
        operation, i.e. the external-attacher.
      * **detachError** ``Optional[VolumeError]`` - The last error encountered during detach operation, if any. This field must
        only be set by the entity completing the detach operation, i.e. the
        external-attacher.
    """
    attached: 'bool'
    attachError: 'Optional[VolumeError]' = None
    attachmentMetadata: 'Optional[dict]' = None
    detachError: 'Optional[VolumeError]' = None


@dataclass
class VolumeError(DictMixin):
    """VolumeError captures an error encountered during a volume operation.

      **parameters**

      * **message** ``Optional[str]`` - String detailing the error encountered during Attach or Detach operation. This
        string may be logged, so it should not contain sensitive information.
      * **time** ``Optional[meta_v1.Time]`` - Time the error was encountered.
    """
    message: 'Optional[str]' = None
    time: 'Optional[meta_v1.Time]' = None


@dataclass
class VolumeNodeResources(DictMixin):
    """VolumeNodeResources is a set of resource limits for scheduling of volumes.

      **parameters**

      * **count** ``Optional[int]`` - Maximum number of unique volumes managed by the CSI driver that can be used on
        a node. A volume that is both attached and mounted on a node is considered to
        be used once, not twice. The same rule applies for a unique volume that is
        shared among multiple pods on the same node. If this field is not specified,
        then the supported number of volumes on this node is unbounded.
    """
    count: 'Optional[int]' = None





# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field



@dataclass
class Info(DictMixin):
    """Info contains versioning information. how we'll want to distribute that
      information.

      **parameters**

      * **buildDate** ``str`` - 
      * **compiler** ``str`` - 
      * **gitCommit** ``str`` - 
      * **gitTreeState** ``str`` - 
      * **gitVersion** ``str`` - 
      * **goVersion** ``str`` - 
      * **major** ``str`` - 
      * **minor** ``str`` - 
      * **platform** ``str`` - 
    """
    buildDate: 'str'
    compiler: 'str'
    gitCommit: 'str'
    gitTreeState: 'str'
    gitVersion: 'str'
    goVersion: 'str'
    major: 'str'
    minor: 'str'
    platform: 'str'





# autogenerated module
from typing import ClassVar

from ..core import resource as res
from ..models import authorization_v1 as m_authorization_v1


class LocalSubjectAccessReview(res.NamespacedResource, m_authorization_v1.LocalSubjectAccessReview):
    """* **Extends**: ``models.authorization_v1.LocalSubjectAccessReview``
       * **Type**: Namespaced Resource
       * **Accepted client methods**: `create`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('authorization.k8s.io', 'v1', 'LocalSubjectAccessReview'),
        plural='localsubjectaccessreviews',
        verbs=['post'],
    )


class SelfSubjectAccessReview(res.GlobalResource, m_authorization_v1.SelfSubjectAccessReview):
    """* **Extends**: ``models.authorization_v1.SelfSubjectAccessReview``
       * **Type**: Global Resource
       * **Accepted client methods**: `create`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('authorization.k8s.io', 'v1', 'SelfSubjectAccessReview'),
        plural='selfsubjectaccessreviews',
        verbs=['post'],
    )


class SelfSubjectRulesReview(res.GlobalResource, m_authorization_v1.SelfSubjectRulesReview):
    """* **Extends**: ``models.authorization_v1.SelfSubjectRulesReview``
       * **Type**: Global Resource
       * **Accepted client methods**: `create`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('authorization.k8s.io', 'v1', 'SelfSubjectRulesReview'),
        plural='selfsubjectrulesreviews',
        verbs=['post'],
    )


class SubjectAccessReview(res.GlobalResource, m_authorization_v1.SubjectAccessReview):
    """* **Extends**: ``models.authorization_v1.SubjectAccessReview``
       * **Type**: Global Resource
       * **Accepted client methods**: `create`
    """
    _api_info = res.ApiInfo(
        resource=res.ResourceDef('authorization.k8s.io', 'v1', 'SubjectAccessReview'),
        plural='subjectaccessreviews',
        verbs=['post'],
    )




try:
    from lightkube.core.schema import DictMixin, field, dataclass

except ImportError:
    # Support for older version of lightkube that lack this class
    from dataclasses import dataclass, field
    from typing import Union, get_type_hints
    from ..core.dataclasses_dict import DataclassDictMixIn as DictMixin, get_args, get_origin
    from ..core import dataclasses_dict

    NoneType = type(None)

    def _remove_optional(tp):
       if get_origin(tp) is Union:
         args = get_args(tp)
         if args[1] is NoneType:
           return args[0]
       return tp

    def _get_type_hints(cl):
        types = get_type_hints(cl)
        return {k: _remove_optional(v) for k, v in types.items()}

    # patch dataclasses_dict so that Optional is removed as it's not supported by old lightkube versions
    dataclasses_dict.get_type_hints = _get_type_hints



# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field

from . import meta_v1
from . import resource


@dataclass
class ContainerResourceMetricSource(DictMixin):
    """ContainerResourceMetricSource indicates how to scale on a resource metric
      known to Kubernetes, as specified in requests and limits, describing each pod
      in the current scale target (e.g. CPU or memory).  The values will be averaged
      together before being compared to the target.  Such metrics are built in to
      Kubernetes, and have special scaling options on top of those available to
      normal per-pod metrics using the "pods" source.  Only one "target" type should
      be set.

      **parameters**

      * **container** ``str`` - container is the name of the container in the pods of the scaling target
      * **name** ``str`` - name is the name of the resource in question.
      * **target** ``MetricTarget`` - target specifies the target value for the given metric
    """
    container: 'str'
    name: 'str'
    target: 'MetricTarget'


@dataclass
class ContainerResourceMetricStatus(DictMixin):
    """ContainerResourceMetricStatus indicates the current value of a resource metric
      known to Kubernetes, as specified in requests and limits, describing a single
      container in each pod in the current scale target (e.g. CPU or memory).  Such
      metrics are built in to Kubernetes, and have special scaling options on top of
      those available to normal per-pod metrics using the "pods" source.

      **parameters**

      * **container** ``str`` - Container is the name of the container in the pods of the scaling target
      * **current** ``MetricValueStatus`` - current contains the current value for the given metric
      * **name** ``str`` - Name is the name of the resource in question.
    """
    container: 'str'
    current: 'MetricValueStatus'
    name: 'str'


@dataclass
class CrossVersionObjectReference(DictMixin):
    """CrossVersionObjectReference contains enough information to let you identify
      the referred resource.

      **parameters**

      * **kind** ``str`` - Kind of the referent; More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
      * **name** ``str`` - Name of the referent; More info:
        http://kubernetes.io/docs/user-guide/identifiers#names
      * **apiVersion** ``Optional[str]`` - API version of the referent
    """
    kind: 'str'
    name: 'str'
    apiVersion: 'Optional[str]' = None


@dataclass
class ExternalMetricSource(DictMixin):
    """ExternalMetricSource indicates how to scale on a metric not associated with
      any Kubernetes object (for example length of queue in cloud messaging service,
      or QPS from loadbalancer running outside of cluster).

      **parameters**

      * **metric** ``MetricIdentifier`` - metric identifies the target metric by name and selector
      * **target** ``MetricTarget`` - target specifies the target value for the given metric
    """
    metric: 'MetricIdentifier'
    target: 'MetricTarget'


@dataclass
class ExternalMetricStatus(DictMixin):
    """ExternalMetricStatus indicates the current value of a global metric not
      associated with any Kubernetes object.

      **parameters**

      * **current** ``MetricValueStatus`` - current contains the current value for the given metric
      * **metric** ``MetricIdentifier`` - metric identifies the target metric by name and selector
    """
    current: 'MetricValueStatus'
    metric: 'MetricIdentifier'


@dataclass
class HPAScalingPolicy(DictMixin):
    """HPAScalingPolicy is a single policy which must hold true for a specified past
      interval.

      **parameters**

      * **periodSeconds** ``int`` - PeriodSeconds specifies the window of time for which the policy should hold
        true. PeriodSeconds must be greater than zero and less than or equal to 1800
        (30 min).
      * **type** ``str`` - Type is used to specify the scaling policy.
      * **value** ``int`` - Value contains the amount of change which is permitted by the policy. It must
        be greater than zero
    """
    periodSeconds: 'int'
    type: 'str'
    value: 'int'


@dataclass
class HPAScalingRules(DictMixin):
    """HPAScalingRules configures the scaling behavior for one direction. These Rules
      are applied after calculating DesiredReplicas from metrics for the HPA. They
      can limit the scaling velocity by specifying scaling policies. They can
      prevent flapping by specifying the stabilization window, so that the number of
      replicas is not set instantly, instead, the safest value from the
      stabilization window is chosen.

      **parameters**

      * **policies** ``Optional[List[HPAScalingPolicy]]`` - policies is a list of potential scaling polices which can be used during
        scaling. At least one policy must be specified, otherwise the HPAScalingRules
        will be discarded as invalid
      * **selectPolicy** ``Optional[str]`` - selectPolicy is used to specify which policy should be used. If not set, the
        default value MaxPolicySelect is used.
      * **stabilizationWindowSeconds** ``Optional[int]`` - StabilizationWindowSeconds is the number of seconds for which past
        recommendations should be considered while scaling up or scaling down.
        StabilizationWindowSeconds must be greater than or equal to zero and less than
        or equal to 3600 (one hour). If not set, use the default values: - For scale
        up: 0 (i.e. no stabilization is done). - For scale down: 300 (i.e. the
        stabilization window is 300 seconds long).
    """
    policies: 'Optional[List[HPAScalingPolicy]]' = None
    selectPolicy: 'Optional[str]' = None
    stabilizationWindowSeconds: 'Optional[int]' = None


@dataclass
class HorizontalPodAutoscaler(DictMixin):
    """HorizontalPodAutoscaler is the configuration for a horizontal pod autoscaler,
      which automatically manages the replica count of any resource implementing the
      scale subresource based on the metrics specified.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - metadata is the standard object metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[HorizontalPodAutoscalerSpec]`` - spec is the specification for the behaviour of the autoscaler. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status.
      * **status** ``Optional[HorizontalPodAutoscalerStatus]`` - status is the current information about the autoscaler.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[HorizontalPodAutoscalerSpec]' = None
    status: 'Optional[HorizontalPodAutoscalerStatus]' = None


@dataclass
class HorizontalPodAutoscalerBehavior(DictMixin):
    """HorizontalPodAutoscalerBehavior configures the scaling behavior of the target
      in both Up and Down directions (scaleUp and scaleDown fields respectively).

      **parameters**

      * **scaleDown** ``Optional[HPAScalingRules]`` - scaleDown is scaling policy for scaling Down. If not set, the default value is
        to allow to scale down to minReplicas pods, with a 300 second stabilization
        window (i.e., the highest recommendation for the last 300sec is used).
      * **scaleUp** ``Optional[HPAScalingRules]`` - scaleUp is scaling policy for scaling Up. If not set, the default value is the
        higher of:
          * increase no more than 4 pods per 60 seconds
          * double the number of pods per 60 seconds
        No stabilization is used.
    """
    scaleDown: 'Optional[HPAScalingRules]' = None
    scaleUp: 'Optional[HPAScalingRules]' = None


@dataclass
class HorizontalPodAutoscalerCondition(DictMixin):
    """HorizontalPodAutoscalerCondition describes the state of a
      HorizontalPodAutoscaler at a certain point.

      **parameters**

      * **status** ``str`` - status is the status of the condition (True, False, Unknown)
      * **type** ``str`` - type describes the current condition
      * **lastTransitionTime** ``Optional[meta_v1.Time]`` - lastTransitionTime is the last time the condition transitioned from one status
        to another
      * **message** ``Optional[str]`` - message is a human-readable explanation containing details about the
        transition
      * **reason** ``Optional[str]`` - reason is the reason for the condition's last transition.
    """
    status: 'str'
    type: 'str'
    lastTransitionTime: 'Optional[meta_v1.Time]' = None
    message: 'Optional[str]' = None
    reason: 'Optional[str]' = None


@dataclass
class HorizontalPodAutoscalerList(DictMixin):
    """HorizontalPodAutoscalerList is a list of horizontal pod autoscaler objects.

      **parameters**

      * **items** ``List[HorizontalPodAutoscaler]`` - items is the list of horizontal pod autoscaler objects.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - metadata is the standard list metadata.
    """
    items: 'List[HorizontalPodAutoscaler]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class HorizontalPodAutoscalerSpec(DictMixin):
    """HorizontalPodAutoscalerSpec describes the desired functionality of the
      HorizontalPodAutoscaler.

      **parameters**

      * **maxReplicas** ``int`` - maxReplicas is the upper limit for the number of replicas to which the
        autoscaler can scale up. It cannot be less that minReplicas.
      * **scaleTargetRef** ``CrossVersionObjectReference`` - scaleTargetRef points to the target resource to scale, and is used to the pods
        for which metrics should be collected, as well as to actually change the
        replica count.
      * **behavior** ``Optional[HorizontalPodAutoscalerBehavior]`` - behavior configures the scaling behavior of the target in both Up and Down
        directions (scaleUp and scaleDown fields respectively). If not set, the
        default HPAScalingRules for scale up and scale down are used.
      * **metrics** ``Optional[List[MetricSpec]]`` - metrics contains the specifications for which to use to calculate the desired
        replica count (the maximum replica count across all metrics will be used).
        The desired replica count is calculated multiplying the ratio between the
        target value and the current value by the current number of pods.  Ergo,
        metrics used must decrease as the pod count is increased, and vice-versa.  See
        the individual metric source types for more information about how each type of
        metric must respond. If not set, the default metric will be set to 80% average
        CPU utilization.
      * **minReplicas** ``Optional[int]`` - minReplicas is the lower limit for the number of replicas to which the
        autoscaler can scale down.  It defaults to 1 pod.  minReplicas is allowed to
        be 0 if the alpha feature gate HPAScaleToZero is enabled and at least one
        Object or External metric is configured.  Scaling is active as long as at
        least one metric value is available.
    """
    maxReplicas: 'int'
    scaleTargetRef: 'CrossVersionObjectReference'
    behavior: 'Optional[HorizontalPodAutoscalerBehavior]' = None
    metrics: 'Optional[List[MetricSpec]]' = None
    minReplicas: 'Optional[int]' = None


@dataclass
class HorizontalPodAutoscalerStatus(DictMixin):
    """HorizontalPodAutoscalerStatus describes the current status of a horizontal pod
      autoscaler.

      **parameters**

      * **currentReplicas** ``int`` - currentReplicas is current number of replicas of pods managed by this
        autoscaler, as last seen by the autoscaler.
      * **desiredReplicas** ``int`` - desiredReplicas is the desired number of replicas of pods managed by this
        autoscaler, as last calculated by the autoscaler.
      * **conditions** ``Optional[List[HorizontalPodAutoscalerCondition]]`` - conditions is the set of conditions required for this autoscaler to scale its
        target, and indicates whether or not those conditions are met.
      * **currentMetrics** ``Optional[List[MetricStatus]]`` - currentMetrics is the last read state of the metrics used by this autoscaler.
      * **lastScaleTime** ``Optional[meta_v1.Time]`` - lastScaleTime is the last time the HorizontalPodAutoscaler scaled the number
        of pods, used by the autoscaler to control how often the number of pods is
        changed.
      * **observedGeneration** ``Optional[int]`` - observedGeneration is the most recent generation observed by this autoscaler.
    """
    currentReplicas: 'int'
    desiredReplicas: 'int'
    conditions: 'Optional[List[HorizontalPodAutoscalerCondition]]' = None
    currentMetrics: 'Optional[List[MetricStatus]]' = None
    lastScaleTime: 'Optional[meta_v1.Time]' = None
    observedGeneration: 'Optional[int]' = None


@dataclass
class MetricIdentifier(DictMixin):
    """MetricIdentifier defines the name and optionally selector for a metric

      **parameters**

      * **name** ``str`` - name is the name of the given metric
      * **selector** ``Optional[meta_v1.LabelSelector]`` - selector is the string-encoded form of a standard kubernetes label selector
        for the given metric When set, it is passed as an additional parameter to the
        metrics server for more specific metrics scoping. When unset, just the
        metricName will be used to gather metrics.
    """
    name: 'str'
    selector: 'Optional[meta_v1.LabelSelector]' = None


@dataclass
class MetricSpec(DictMixin):
    """MetricSpec specifies how to scale based on a single metric (only `type` and
      one other matching field should be set at once).

      **parameters**

      * **type** ``str`` - type is the type of metric source.  It should be one of "ContainerResource",
        "External", "Object", "Pods" or "Resource", each mapping to a matching field
        in the object. Note: "ContainerResource" type is available on when the
        feature-gate HPAContainerMetrics is enabled
      * **containerResource** ``Optional[ContainerResourceMetricSource]`` - container resource refers to a resource metric (such as those specified in
        requests and limits) known to Kubernetes describing a single container in each
        pod of the current scale target (e.g. CPU or memory). Such metrics are built
        in to Kubernetes, and have special scaling options on top of those available
        to normal per-pod metrics using the "pods" source. This is an alpha feature
        and can be enabled by the HPAContainerMetrics feature flag.
      * **external** ``Optional[ExternalMetricSource]`` - external refers to a global metric that is not associated with any Kubernetes
        object. It allows autoscaling based on information coming from components
        running outside of cluster (for example length of queue in cloud messaging
        service, or QPS from loadbalancer running outside of cluster).
      * **object** ``Optional[ObjectMetricSource]`` - object refers to a metric describing a single kubernetes object (for example,
        hits-per-second on an Ingress object).
      * **pods** ``Optional[PodsMetricSource]`` - pods refers to a metric describing each pod in the current scale target (for
        example, transactions-processed-per-second).  The values will be averaged
        together before being compared to the target value.
      * **resource** ``Optional[ResourceMetricSource]`` - resource refers to a resource metric (such as those specified in requests and
        limits) known to Kubernetes describing each pod in the current scale target
        (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have
        special scaling options on top of those available to normal per-pod metrics
        using the "pods" source.
    """
    type: 'str'
    containerResource: 'Optional[ContainerResourceMetricSource]' = None
    external: 'Optional[ExternalMetricSource]' = None
    object: 'Optional[ObjectMetricSource]' = None
    pods: 'Optional[PodsMetricSource]' = None
    resource: 'Optional[ResourceMetricSource]' = None


@dataclass
class MetricStatus(DictMixin):
    """MetricStatus describes the last-read state of a single metric.

      **parameters**

      * **type** ``str`` - type is the type of metric source.  It will be one of "ContainerResource",
        "External", "Object", "Pods" or "Resource", each corresponds to a matching
        field in the object. Note: "ContainerResource" type is available on when the
        feature-gate HPAContainerMetrics is enabled
      * **containerResource** ``Optional[ContainerResourceMetricStatus]`` - container resource refers to a resource metric (such as those specified in
        requests and limits) known to Kubernetes describing a single container in each
        pod in the current scale target (e.g. CPU or memory). Such metrics are built
        in to Kubernetes, and have special scaling options on top of those available
        to normal per-pod metrics using the "pods" source.
      * **external** ``Optional[ExternalMetricStatus]`` - external refers to a global metric that is not associated with any Kubernetes
        object. It allows autoscaling based on information coming from components
        running outside of cluster (for example length of queue in cloud messaging
        service, or QPS from loadbalancer running outside of cluster).
      * **object** ``Optional[ObjectMetricStatus]`` - object refers to a metric describing a single kubernetes object (for example,
        hits-per-second on an Ingress object).
      * **pods** ``Optional[PodsMetricStatus]`` - pods refers to a metric describing each pod in the current scale target (for
        example, transactions-processed-per-second).  The values will be averaged
        together before being compared to the target value.
      * **resource** ``Optional[ResourceMetricStatus]`` - resource refers to a resource metric (such as those specified in requests and
        limits) known to Kubernetes describing each pod in the current scale target
        (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have
        special scaling options on top of those available to normal per-pod metrics
        using the "pods" source.
    """
    type: 'str'
    containerResource: 'Optional[ContainerResourceMetricStatus]' = None
    external: 'Optional[ExternalMetricStatus]' = None
    object: 'Optional[ObjectMetricStatus]' = None
    pods: 'Optional[PodsMetricStatus]' = None
    resource: 'Optional[ResourceMetricStatus]' = None


@dataclass
class MetricTarget(DictMixin):
    """MetricTarget defines the target value, average value, or average utilization
      of a specific metric

      **parameters**

      * **type** ``str`` - type represents whether the metric type is Utilization, Value, or AverageValue
      * **averageUtilization** ``Optional[int]`` - averageUtilization is the target value of the average of the resource metric
        across all relevant pods, represented as a percentage of the requested value
        of the resource for the pods. Currently only valid for Resource metric source
        type
      * **averageValue** ``Optional[resource.Quantity]`` - averageValue is the target value of the average of the metric across all
        relevant pods (as a quantity)
      * **value** ``Optional[resource.Quantity]`` - value is the target value of the metric (as a quantity).
    """
    type: 'str'
    averageUtilization: 'Optional[int]' = None
    averageValue: 'Optional[resource.Quantity]' = None
    value: 'Optional[resource.Quantity]' = None


@dataclass
class MetricValueStatus(DictMixin):
    """MetricValueStatus holds the current value for a metric

      **parameters**

      * **averageUtilization** ``Optional[int]`` - currentAverageUtilization is the current value of the average of the resource
        metric across all relevant pods, represented as a percentage of the requested
        value of the resource for the pods.
      * **averageValue** ``Optional[resource.Quantity]`` - averageValue is the current value of the average of the metric across all
        relevant pods (as a quantity)
      * **value** ``Optional[resource.Quantity]`` - value is the current value of the metric (as a quantity).
    """
    averageUtilization: 'Optional[int]' = None
    averageValue: 'Optional[resource.Quantity]' = None
    value: 'Optional[resource.Quantity]' = None


@dataclass
class ObjectMetricSource(DictMixin):
    """ObjectMetricSource indicates how to scale on a metric describing a kubernetes
      object (for example, hits-per-second on an Ingress object).

      **parameters**

      * **describedObject** ``CrossVersionObjectReference`` - 
      * **metric** ``MetricIdentifier`` - metric identifies the target metric by name and selector
      * **target** ``MetricTarget`` - target specifies the target value for the given metric
    """
    describedObject: 'CrossVersionObjectReference'
    metric: 'MetricIdentifier'
    target: 'MetricTarget'


@dataclass
class ObjectMetricStatus(DictMixin):
    """ObjectMetricStatus indicates the current value of a metric describing a
      kubernetes object (for example, hits-per-second on an Ingress object).

      **parameters**

      * **current** ``MetricValueStatus`` - current contains the current value for the given metric
      * **describedObject** ``CrossVersionObjectReference`` - 
      * **metric** ``MetricIdentifier`` - metric identifies the target metric by name and selector
    """
    current: 'MetricValueStatus'
    describedObject: 'CrossVersionObjectReference'
    metric: 'MetricIdentifier'


@dataclass
class PodsMetricSource(DictMixin):
    """PodsMetricSource indicates how to scale on a metric describing each pod in the
      current scale target (for example, transactions-processed-per-second). The
      values will be averaged together before being compared to the target value.

      **parameters**

      * **metric** ``MetricIdentifier`` - metric identifies the target metric by name and selector
      * **target** ``MetricTarget`` - target specifies the target value for the given metric
    """
    metric: 'MetricIdentifier'
    target: 'MetricTarget'


@dataclass
class PodsMetricStatus(DictMixin):
    """PodsMetricStatus indicates the current value of a metric describing each pod
      in the current scale target (for example, transactions-processed-per-second).

      **parameters**

      * **current** ``MetricValueStatus`` - current contains the current value for the given metric
      * **metric** ``MetricIdentifier`` - metric identifies the target metric by name and selector
    """
    current: 'MetricValueStatus'
    metric: 'MetricIdentifier'


@dataclass
class ResourceMetricSource(DictMixin):
    """ResourceMetricSource indicates how to scale on a resource metric known to
      Kubernetes, as specified in requests and limits, describing each pod in the
      current scale target (e.g. CPU or memory).  The values will be averaged
      together before being compared to the target.  Such metrics are built in to
      Kubernetes, and have special scaling options on top of those available to
      normal per-pod metrics using the "pods" source.  Only one "target" type should
      be set.

      **parameters**

      * **name** ``str`` - name is the name of the resource in question.
      * **target** ``MetricTarget`` - target specifies the target value for the given metric
    """
    name: 'str'
    target: 'MetricTarget'


@dataclass
class ResourceMetricStatus(DictMixin):
    """ResourceMetricStatus indicates the current value of a resource metric known to
      Kubernetes, as specified in requests and limits, describing each pod in the
      current scale target (e.g. CPU or memory).  Such metrics are built in to
      Kubernetes, and have special scaling options on top of those available to
      normal per-pod metrics using the "pods" source.

      **parameters**

      * **current** ``MetricValueStatus`` - current contains the current value for the given metric
      * **name** ``str`` - Name is the name of the resource in question.
    """
    current: 'MetricValueStatus'
    name: 'str'





from setuptools import setup
from pathlib import Path

from lightkube.models import __version__


setup(
    name='lightkube-models',
    version=__version__,
    description='Models and Resources for lightkube module',
    long_description=Path("README.md").read_text(),
    long_description_content_type="text/markdown",
    author='Giuseppe Tribulato',
    author_email='gtsystem@gmail.com',
    license='Apache Software License',
    url='https://github.com/gtsystem/lightkube-models',
    packages=['lightkube.models', 'lightkube.resources'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ]
)



# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field

from . import meta_v1


@dataclass
class APIService(DictMixin):
    """APIService represents a server for a particular GroupVersion. Name must be
      "version.group".

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[APIServiceSpec]`` - Spec contains information for locating and communicating with a server
      * **status** ``Optional[APIServiceStatus]`` - Status contains derived information about an API server
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[APIServiceSpec]' = None
    status: 'Optional[APIServiceStatus]' = None


@dataclass
class APIServiceCondition(DictMixin):
    """APIServiceCondition describes the state of an APIService at a particular point

      **parameters**

      * **status** ``str`` - Status is the status of the condition. Can be True, False, Unknown.
      * **type** ``str`` - Type is the type of the condition.
      * **lastTransitionTime** ``Optional[meta_v1.Time]`` - Last time the condition transitioned from one status to another.
      * **message** ``Optional[str]`` - Human-readable message indicating details about last transition.
      * **reason** ``Optional[str]`` - Unique, one-word, CamelCase reason for the condition's last transition.
    """
    status: 'str'
    type: 'str'
    lastTransitionTime: 'Optional[meta_v1.Time]' = None
    message: 'Optional[str]' = None
    reason: 'Optional[str]' = None


@dataclass
class APIServiceList(DictMixin):
    """APIServiceList is a list of APIService objects.

      **parameters**

      * **items** ``List[APIService]`` - Items is the list of APIService
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[APIService]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class APIServiceSpec(DictMixin):
    """APIServiceSpec contains information for locating and communicating with a
      server. Only https is supported, though you are able to disable certificate
      verification.

      **parameters**

      * **groupPriorityMinimum** ``int`` - GroupPriorityMininum is the priority this group should have at least. Higher
        priority means that the group is preferred by clients over lower priority
        ones. Note that other versions of this group might specify even higher
        GroupPriorityMininum values such that the whole group gets a higher priority.
        The primary sort is based on GroupPriorityMinimum, ordered highest number to
        lowest (20 before 10). The secondary sort is based on the alphabetical
        comparison of the name of the object.  (v1.bar before v1.foo) We'd recommend
        something like: *.k8s.io (except extensions) at 18000 and PaaSes (OpenShift,
        Deis) are recommended to be in the 2000s
      * **versionPriority** ``int`` - VersionPriority controls the ordering of this API version inside of its group.
        Must be greater than zero. The primary sort is based on VersionPriority,
        ordered highest to lowest (20 before 10). Since it's inside of a group, the
        number can be small, probably in the 10s. In case of equal version priorities,
        the version string will be used to compute the order inside a group. If the
        version string is "kube-like", it will sort above non "kube-like" version
        strings, which are ordered lexicographically. "Kube-like" versions start with
        a "v", then are followed by a number (the major version), then optionally the
        string "alpha" or "beta" and another number (the minor version). These are
        sorted first by GA > beta > alpha (where GA is a version with no suffix such
        as beta or alpha), and then by comparing major version, then minor version. An
        example sorted list of versions: v10, v2, v1, v11beta2, v10beta3, v3beta1,
        v12alpha1, v11alpha2, foo1, foo10.
      * **caBundle** ``Optional[str]`` - CABundle is a PEM encoded CA bundle which will be used to validate an API
        server's serving certificate. If unspecified, system trust roots on the
        apiserver are used.
      * **group** ``Optional[str]`` - Group is the API group name this server hosts
      * **insecureSkipTLSVerify** ``Optional[bool]`` - InsecureSkipTLSVerify disables TLS certificate verification when communicating
        with this server. This is strongly discouraged.  You should use the CABundle
        instead.
      * **service** ``Optional[ServiceReference]`` - Service is a reference to the service for this API server.  It must
        communicate on port 443. If the Service is nil, that means the handling for
        the API groupversion is handled locally on this server. The call will simply
        delegate to the normal handler chain to be fulfilled.
      * **version** ``Optional[str]`` - Version is the API version this server hosts.  For example, "v1"
    """
    groupPriorityMinimum: 'int'
    versionPriority: 'int'
    caBundle: 'Optional[str]' = None
    group: 'Optional[str]' = None
    insecureSkipTLSVerify: 'Optional[bool]' = None
    service: 'Optional[ServiceReference]' = None
    version: 'Optional[str]' = None


@dataclass
class APIServiceStatus(DictMixin):
    """APIServiceStatus contains derived information about an API server

      **parameters**

      * **conditions** ``Optional[List[APIServiceCondition]]`` - Current service state of apiService.
    """
    conditions: 'Optional[List[APIServiceCondition]]' = None


@dataclass
class ServiceReference(DictMixin):
    """ServiceReference holds a reference to Service.legacy.k8s.io

      **parameters**

      * **name** ``Optional[str]`` - Name is the name of the service
      * **namespace** ``Optional[str]`` - Namespace is the namespace of the service
      * **port** ``Optional[int]`` - If specified, the port on the service that hosting webhook. Default to 443 for
        backward compatibility. `port` should be a valid port number (1-65535,
        inclusive).
    """
    name: 'Optional[str]' = None
    namespace: 'Optional[str]' = None
    port: 'Optional[int]' = None





# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field

from . import core_v1
from . import util_intstr
from . import meta_v1


@dataclass
class HTTPIngressPath(DictMixin):
    """HTTPIngressPath associates a path with a backend. Incoming urls matching the
      path are forwarded to the backend.

      **parameters**

      * **backend** ``IngressBackend`` - Backend defines the referenced service endpoint to which the traffic will be
        forwarded to.
      * **pathType** ``str`` - PathType determines the interpretation of the Path matching. PathType can be
        one of the following values: * Exact: Matches the URL path exactly. * Prefix:
        Matches based on a URL path prefix split by '/'. Matching is
          done on a path element by element basis. A path element refers is the
          list of labels in the path split by the '/' separator. A request is a
          match for path p if every p is an element-wise prefix of p of the
          request path. Note that if the last element of the path is a substring
          of the last element in request path, it is not a match (e.g. /foo/bar
          matches /foo/bar/baz, but does not match /foo/barbaz).
        * ImplementationSpecific: Interpretation of the Path matching is up to
          the IngressClass. Implementations can treat this as a separate PathType
          or treat it identically to Prefix or Exact path types.
        Implementations are required to support all path types.
      * **path** ``Optional[str]`` - Path is matched against the path of an incoming request. Currently it can
        contain characters disallowed from the conventional "path" part of a URL as
        defined by RFC 3986. Paths must begin with a '/' and must be present when
        using PathType with value "Exact" or "Prefix".
    """
    backend: 'IngressBackend'
    pathType: 'str'
    path: 'Optional[str]' = None


@dataclass
class HTTPIngressRuleValue(DictMixin):
    """HTTPIngressRuleValue is a list of http selectors pointing to backends. In the
      example: http://<host>/<path>?<searchpart> -> backend where where parts of the
      url correspond to RFC 3986, this resource will be used to match against
      everything after the last '/' and before the first '?' or '#'.

      **parameters**

      * **paths** ``List[HTTPIngressPath]`` - A collection of paths that map requests to backends.
    """
    paths: 'List[HTTPIngressPath]'


@dataclass
class IPBlock(DictMixin):
    """IPBlock describes a particular CIDR (Ex. "192.168.1.1/24","2001:db9::/64")
      that is allowed to the pods matched by a NetworkPolicySpec's podSelector. The
      except entry describes CIDRs that should not be included within this rule.

      **parameters**

      * **cidr** ``str`` - CIDR is a string representing the IP Block Valid examples are "192.168.1.1/24"
        or "2001:db9::/64"
      * **except_** ``Optional[List[str]]`` - Except is a slice of CIDRs that should not be included within an IP Block
        Valid examples are "192.168.1.1/24" or "2001:db9::/64" Except values will be
        rejected if they are outside the CIDR range
    """
    cidr: 'str'
    except_: 'Optional[List[str]]' = field(metadata={"json": "except"}, default=None)


@dataclass
class Ingress(DictMixin):
    """Ingress is a collection of rules that allow inbound connections to reach the
      endpoints defined by a backend. An Ingress can be configured to give services
      externally-reachable urls, load balance traffic, terminate SSL, offer name
      based virtual hosting etc.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[IngressSpec]`` - Spec is the desired state of the Ingress. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
      * **status** ``Optional[IngressStatus]`` - Status is the current state of the Ingress. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[IngressSpec]' = None
    status: 'Optional[IngressStatus]' = None


@dataclass
class IngressBackend(DictMixin):
    """IngressBackend describes all endpoints for a given service and port.

      **parameters**

      * **resource** ``Optional[core_v1.TypedLocalObjectReference]`` - Resource is an ObjectRef to another Kubernetes resource in the namespace of
        the Ingress object. If resource is specified, a service.Name and service.Port
        must not be specified. This is a mutually exclusive setting with "Service".
      * **service** ``Optional[IngressServiceBackend]`` - Service references a Service as a Backend. This is a mutually exclusive
        setting with "Resource".
    """
    resource: 'Optional[core_v1.TypedLocalObjectReference]' = None
    service: 'Optional[IngressServiceBackend]' = None


@dataclass
class IngressClass(DictMixin):
    """IngressClass represents the class of the Ingress, referenced by the Ingress
      Spec. The `ingressclass.kubernetes.io/is-default-class` annotation can be used
      to indicate that an IngressClass should be considered default. When a single
      IngressClass resource has this annotation set to true, new Ingress resources
      without a class specified will be assigned this default class.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[IngressClassSpec]`` - Spec is the desired state of the IngressClass. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[IngressClassSpec]' = None


@dataclass
class IngressClassList(DictMixin):
    """IngressClassList is a collection of IngressClasses.

      **parameters**

      * **items** ``List[IngressClass]`` - Items is the list of IngressClasses.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata.
    """
    items: 'List[IngressClass]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class IngressClassParametersReference(DictMixin):
    """IngressClassParametersReference identifies an API object. This can be used to
      specify a cluster or namespace-scoped resource.

      **parameters**

      * **kind** ``str`` - Kind is the type of resource being referenced.
      * **name** ``str`` - Name is the name of resource being referenced.
      * **apiGroup** ``Optional[str]`` - APIGroup is the group for the resource being referenced. If APIGroup is not
        specified, the specified Kind must be in the core API group. For any other
        third-party types, APIGroup is required.
      * **namespace** ``Optional[str]`` - Namespace is the namespace of the resource being referenced. This field is
        required when scope is set to "Namespace" and must be unset when scope is set
        to "Cluster".
      * **scope** ``Optional[str]`` - Scope represents if this refers to a cluster or namespace scoped resource.
        This may be set to "Cluster" (default) or "Namespace".
    """
    kind: 'str'
    name: 'str'
    apiGroup: 'Optional[str]' = None
    namespace: 'Optional[str]' = None
    scope: 'Optional[str]' = None


@dataclass
class IngressClassSpec(DictMixin):
    """IngressClassSpec provides information about the class of an Ingress.

      **parameters**

      * **controller** ``Optional[str]`` - Controller refers to the name of the controller that should handle this class.
        This allows for different "flavors" that are controlled by the same
        controller. For example, you may have different Parameters for the same
        implementing controller. This should be specified as a domain-prefixed path no
        more than 250 characters in length, e.g. "acme.io/ingress-controller". This
        field is immutable.
      * **parameters** ``Optional[IngressClassParametersReference]`` - Parameters is a link to a custom resource containing additional configuration
        for the controller. This is optional if the controller does not require extra
        parameters.
    """
    controller: 'Optional[str]' = None
    parameters: 'Optional[IngressClassParametersReference]' = None


@dataclass
class IngressList(DictMixin):
    """IngressList is a collection of Ingress.

      **parameters**

      * **items** ``List[Ingress]`` - Items is the list of Ingress.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[Ingress]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class IngressRule(DictMixin):
    """IngressRule represents the rules mapping the paths under a specified host to
      the related backend services. Incoming requests are first evaluated for a host
      match, then routed to the backend associated with the matching
      IngressRuleValue.

      **parameters**

      * **host** ``Optional[str]`` - Host is the fully qualified domain name of a network host, as defined by RFC
        3986. Note the following deviations from the "host" part of the URI as defined
        in RFC 3986: 1. IPs are not allowed. Currently an IngressRuleValue can only
        apply to
           the IP in the Spec of the parent Ingress.
        2. The `:` delimiter is not respected because ports are not allowed.
        	  Currently the port of an Ingress is implicitly :80 for http and
        	  :443 for https.
        Both these may change in the future. Incoming requests are matched against the
        host before the IngressRuleValue. If the host is unspecified, the Ingress
        routes all traffic based on the specified IngressRuleValue.
        Host can be "precise" which is a domain name without the terminating dot of a
        network host (e.g. "foo.bar.com") or "wildcard", which is a domain name
        prefixed with a single wildcard label (e.g. "*.foo.com"). The wildcard
        character '*' must appear by itself as the first DNS label and matches only a
        single label. You cannot have a wildcard label by itself (e.g. Host == "*").
        Requests will be matched against the Host field in the following way: 1. If
        Host is precise, the request matches this rule if the http host header is
        equal to Host. 2. If Host is a wildcard, then the request matches this rule if
        the http host header is to equal to the suffix (removing the first label) of
        the wildcard rule.
      * **http** ``Optional[HTTPIngressRuleValue]`` - 
    """
    host: 'Optional[str]' = None
    http: 'Optional[HTTPIngressRuleValue]' = None


@dataclass
class IngressServiceBackend(DictMixin):
    """IngressServiceBackend references a Kubernetes Service as a Backend.

      **parameters**

      * **name** ``str`` - Name is the referenced service. The service must exist in the same namespace
        as the Ingress object.
      * **port** ``Optional[ServiceBackendPort]`` - Port of the referenced service. A port name or port number is required for a
        IngressServiceBackend.
    """
    name: 'str'
    port: 'Optional[ServiceBackendPort]' = None


@dataclass
class IngressSpec(DictMixin):
    """IngressSpec describes the Ingress the user wishes to exist.

      **parameters**

      * **defaultBackend** ``Optional[IngressBackend]`` - DefaultBackend is the backend that should handle requests that don't match any
        rule. If Rules are not specified, DefaultBackend must be specified. If
        DefaultBackend is not set, the handling of requests that do not match any of
        the rules will be up to the Ingress controller.
      * **ingressClassName** ``Optional[str]`` - IngressClassName is the name of an IngressClass cluster resource. Ingress
        controller implementations use this field to know whether they should be
        serving this Ingress resource, by a transitive connection (controller ->
        IngressClass -> Ingress resource). Although the `kubernetes.io/ingress.class`
        annotation (simple constant name) was never formally defined, it was widely
        supported by Ingress controllers to create a direct binding between Ingress
        controller and Ingress resources. Newly created Ingress resources should
        prefer using the field. However, even though the annotation is officially
        deprecated, for backwards compatibility reasons, ingress controllers should
        still honor that annotation if present.
      * **rules** ``Optional[List[IngressRule]]`` - A list of host rules used to configure the Ingress. If unspecified, or no rule
        matches, all traffic is sent to the default backend.
      * **tls** ``Optional[List[IngressTLS]]`` - TLS configuration. Currently the Ingress only supports a single TLS port, 443.
        If multiple members of this list specify different hosts, they will be
        multiplexed on the same port according to the hostname specified through the
        SNI TLS extension, if the ingress controller fulfilling the ingress supports
        SNI.
    """
    defaultBackend: 'Optional[IngressBackend]' = None
    ingressClassName: 'Optional[str]' = None
    rules: 'Optional[List[IngressRule]]' = None
    tls: 'Optional[List[IngressTLS]]' = None


@dataclass
class IngressStatus(DictMixin):
    """IngressStatus describe the current state of the Ingress.

      **parameters**

      * **loadBalancer** ``Optional[core_v1.LoadBalancerStatus]`` - LoadBalancer contains the current status of the load-balancer.
    """
    loadBalancer: 'Optional[core_v1.LoadBalancerStatus]' = None


@dataclass
class IngressTLS(DictMixin):
    """IngressTLS describes the transport layer security associated with an Ingress.

      **parameters**

      * **hosts** ``Optional[List[str]]`` - Hosts are a list of hosts included in the TLS certificate. The values in this
        list must match the name/s used in the tlsSecret. Defaults to the wildcard
        host setting for the loadbalancer controller fulfilling this Ingress, if left
        unspecified.
      * **secretName** ``Optional[str]`` - SecretName is the name of the secret used to terminate TLS traffic on port
        443. Field is left optional to allow TLS routing based on SNI hostname alone.
        If the SNI host in a listener conflicts with the "Host" header field used by
        an IngressRule, the SNI host is used for termination and value of the Host
        header is used for routing.
    """
    hosts: 'Optional[List[str]]' = None
    secretName: 'Optional[str]' = None


@dataclass
class NetworkPolicy(DictMixin):
    """NetworkPolicy describes what network traffic is allowed for a set of Pods

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object's metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
      * **spec** ``Optional[NetworkPolicySpec]`` - Specification of the desired behavior for this NetworkPolicy.
      * **status** ``Optional[NetworkPolicyStatus]`` - Status is the current state of the NetworkPolicy. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    spec: 'Optional[NetworkPolicySpec]' = None
    status: 'Optional[NetworkPolicyStatus]' = None


@dataclass
class NetworkPolicyEgressRule(DictMixin):
    """NetworkPolicyEgressRule describes a particular set of traffic that is allowed
      out of pods matched by a NetworkPolicySpec's podSelector. The traffic must
      match both ports and to. This type is beta-level in 1.8

      **parameters**

      * **ports** ``Optional[List[NetworkPolicyPort]]`` - List of destination ports for outgoing traffic. Each item in this list is
        combined using a logical OR. If this field is empty or missing, this rule
        matches all ports (traffic not restricted by port). If this field is present
        and contains at least one item, then this rule allows traffic only if the
        traffic matches at least one port in the list.
      * **to** ``Optional[List[NetworkPolicyPeer]]`` - List of destinations for outgoing traffic of pods selected for this rule.
        Items in this list are combined using a logical OR operation. If this field is
        empty or missing, this rule matches all destinations (traffic not restricted
        by destination). If this field is present and contains at least one item, this
        rule allows traffic only if the traffic matches at least one item in the to
        list.
    """
    ports: 'Optional[List[NetworkPolicyPort]]' = None
    to: 'Optional[List[NetworkPolicyPeer]]' = None


@dataclass
class NetworkPolicyIngressRule(DictMixin):
    """NetworkPolicyIngressRule describes a particular set of traffic that is allowed
      to the pods matched by a NetworkPolicySpec's podSelector. The traffic must
      match both ports and from.

      **parameters**

      * **from_** ``Optional[List[NetworkPolicyPeer]]`` - List of sources which should be able to access the pods selected for this
        rule. Items in this list are combined using a logical OR operation. If this
        field is empty or missing, this rule matches all sources (traffic not
        restricted by source). If this field is present and contains at least one
        item, this rule allows traffic only if the traffic matches at least one item
        in the from list.
      * **ports** ``Optional[List[NetworkPolicyPort]]`` - List of ports which should be made accessible on the pods selected for this
        rule. Each item in this list is combined using a logical OR. If this field is
        empty or missing, this rule matches all ports (traffic not restricted by
        port). If this field is present and contains at least one item, then this rule
        allows traffic only if the traffic matches at least one port in the list.
    """
    from_: 'Optional[List[NetworkPolicyPeer]]' = field(metadata={"json": "from"}, default=None)
    ports: 'Optional[List[NetworkPolicyPort]]' = None


@dataclass
class NetworkPolicyList(DictMixin):
    """NetworkPolicyList is a list of NetworkPolicy objects.

      **parameters**

      * **items** ``List[NetworkPolicy]`` - Items is a list of schema objects.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    items: 'List[NetworkPolicy]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class NetworkPolicyPeer(DictMixin):
    """NetworkPolicyPeer describes a peer to allow traffic to/from. Only certain
      combinations of fields are allowed

      **parameters**

      * **ipBlock** ``Optional[IPBlock]`` - IPBlock defines policy on a particular IPBlock. If this field is set then
        neither of the other fields can be.
      * **namespaceSelector** ``Optional[meta_v1.LabelSelector]`` - Selects Namespaces using cluster-scoped labels. This field follows standard
        label selector semantics; if present but empty, it selects all namespaces.
        If PodSelector is also set, then the NetworkPolicyPeer as a whole selects the
        Pods matching PodSelector in the Namespaces selected by NamespaceSelector.
        Otherwise it selects all Pods in the Namespaces selected by NamespaceSelector.
      * **podSelector** ``Optional[meta_v1.LabelSelector]`` - This is a label selector which selects Pods. This field follows standard label
        selector semantics; if present but empty, it selects all pods.
        If NamespaceSelector is also set, then the NetworkPolicyPeer as a whole
        selects the Pods matching PodSelector in the Namespaces selected by
        NamespaceSelector. Otherwise it selects the Pods matching PodSelector in the
        policy's own Namespace.
    """
    ipBlock: 'Optional[IPBlock]' = None
    namespaceSelector: 'Optional[meta_v1.LabelSelector]' = None
    podSelector: 'Optional[meta_v1.LabelSelector]' = None


@dataclass
class NetworkPolicyPort(DictMixin):
    """NetworkPolicyPort describes a port to allow traffic on

      **parameters**

      * **endPort** ``Optional[int]`` - If set, indicates that the range of ports from port to endPort, inclusive,
        should be allowed by the policy. This field cannot be defined if the port
        field is not defined or if the port field is defined as a named (string) port.
        The endPort must be equal or greater than port.
      * **port** ``Optional[util_intstr.IntOrString]`` - The port on the given protocol. This can either be a numerical or named port
        on a pod. If this field is not provided, this matches all port names and
        numbers. If present, only traffic on the specified protocol AND port will be
        matched.
      * **protocol** ``Optional[str]`` - The protocol (TCP, UDP, or SCTP) which traffic must match. If not specified,
        this field defaults to TCP.
    """
    endPort: 'Optional[int]' = None
    port: 'Optional[util_intstr.IntOrString]' = None
    protocol: 'Optional[str]' = None


@dataclass
class NetworkPolicySpec(DictMixin):
    """NetworkPolicySpec provides the specification of a NetworkPolicy

      **parameters**

      * **podSelector** ``meta_v1.LabelSelector`` - Selects the pods to which this NetworkPolicy object applies. The array of
        ingress rules is applied to any pods selected by this field. Multiple network
        policies can select the same set of pods. In this case, the ingress rules for
        each are combined additively. This field is NOT optional and follows standard
        label selector semantics. An empty podSelector matches all pods in this
        namespace.
      * **egress** ``Optional[List[NetworkPolicyEgressRule]]`` - List of egress rules to be applied to the selected pods. Outgoing traffic is
        allowed if there are no NetworkPolicies selecting the pod (and cluster policy
        otherwise allows the traffic), OR if the traffic matches at least one egress
        rule across all of the NetworkPolicy objects whose podSelector matches the
        pod. If this field is empty then this NetworkPolicy limits all outgoing
        traffic (and serves solely to ensure that the pods it selects are isolated by
        default). This field is beta-level in 1.8
      * **ingress** ``Optional[List[NetworkPolicyIngressRule]]`` - List of ingress rules to be applied to the selected pods. Traffic is allowed
        to a pod if there are no NetworkPolicies selecting the pod (and cluster policy
        otherwise allows the traffic), OR if the traffic source is the pod's local
        node, OR if the traffic matches at least one ingress rule across all of the
        NetworkPolicy objects whose podSelector matches the pod. If this field is
        empty then this NetworkPolicy does not allow any traffic (and serves solely to
        ensure that the pods it selects are isolated by default)
      * **policyTypes** ``Optional[List[str]]`` - List of rule types that the NetworkPolicy relates to. Valid options are
        ["Ingress"], ["Egress"], or ["Ingress", "Egress"]. If this field is not
        specified, it will default based on the existence of Ingress or Egress rules;
        policies that contain an Egress section are assumed to affect Egress, and all
        policies (whether or not they contain an Ingress section) are assumed to
        affect Ingress. If you want to write an egress-only policy, you must
        explicitly specify policyTypes [ "Egress" ]. Likewise, if you want to write a
        policy that specifies that no egress is allowed, you must specify a
        policyTypes value that include "Egress" (since such a policy would not include
        an Egress section and would otherwise default to just [ "Ingress" ]). This
        field is beta-level in 1.8
    """
    podSelector: 'meta_v1.LabelSelector'
    egress: 'Optional[List[NetworkPolicyEgressRule]]' = None
    ingress: 'Optional[List[NetworkPolicyIngressRule]]' = None
    policyTypes: 'Optional[List[str]]' = None


@dataclass
class NetworkPolicyStatus(DictMixin):
    """NetworkPolicyStatus describe the current state of the NetworkPolicy.

      **parameters**

      * **conditions** ``Optional[List[meta_v1.Condition]]`` - Conditions holds an array of metav1.Condition that describe the state of the
        NetworkPolicy. Current service state
    """
    conditions: 'Optional[List[meta_v1.Condition]]' = None


@dataclass
class ServiceBackendPort(DictMixin):
    """ServiceBackendPort is the service port being referenced.

      **parameters**

      * **name** ``Optional[str]`` - Name is the name of the port on the Service. This is a mutually exclusive
        setting with "Number".
      * **number** ``Optional[int]`` - Number is the numerical port number (e.g. 80) on the Service. This is a
        mutually exclusive setting with "Name".
    """
    name: 'Optional[str]' = None
    number: 'Optional[int]' = None





# autogenerated module
from typing import List, Optional, TYPE_CHECKING

from ._schema import dataclass, field, DictMixin

if TYPE_CHECKING:   # Fix for pycharm autocompletion https://youtrack.jetbrains.com/issue/PY-54560
    from dataclasses import dataclass, field

from . import meta_v1


@dataclass
class MutatingWebhook(DictMixin):
    """MutatingWebhook describes an admission webhook and the resources and
      operations it applies to.

      **parameters**

      * **admissionReviewVersions** ``List[str]`` - AdmissionReviewVersions is an ordered list of preferred `AdmissionReview`
        versions the Webhook expects. API server will try to use first version in the
        list which it supports. If none of the versions specified in this list
        supported by API server, validation will fail for this object. If a persisted
        webhook configuration specifies allowed versions and does not include any
        versions known to the API Server, calls to the webhook will fail and be
        subject to the failure policy.
      * **clientConfig** ``WebhookClientConfig`` - ClientConfig defines how to communicate with the hook. Required
      * **name** ``str`` - The name of the admission webhook. Name should be fully qualified, e.g.,
        imagepolicy.kubernetes.io, where "imagepolicy" is the name of the webhook, and
        kubernetes.io is the name of the organization. Required.
      * **sideEffects** ``str`` - SideEffects states whether this webhook has side effects. Acceptable values
        are: None, NoneOnDryRun (webhooks created via v1beta1 may also specify Some or
        Unknown). Webhooks with side effects MUST implement a reconciliation system,
        since a request may be rejected by a future step in the admission chain and
        the side effects therefore need to be undone. Requests with the dryRun
        attribute will be auto-rejected if they match a webhook with sideEffects ==
        Unknown or Some.
      * **failurePolicy** ``Optional[str]`` - FailurePolicy defines how unrecognized errors from the admission endpoint are
        handled - allowed values are Ignore or Fail. Defaults to Fail.
      * **matchPolicy** ``Optional[str]`` - matchPolicy defines how the "rules" list is used to match incoming requests.
        Allowed values are "Exact" or "Equivalent".
        - Exact: match a request only if it exactly matches a specified rule. For
        example, if deployments can be modified via apps/v1, apps/v1beta1, and
        extensions/v1beta1, but "rules" only included `apiGroups:["apps"],
        apiVersions:["v1"], resources: ["deployments"]`, a request to apps/v1beta1 or
        extensions/v1beta1 would not be sent to the webhook.
        - Equivalent: match a request if modifies a resource listed in rules, even via
        another API group or version. For example, if deployments can be modified via
        apps/v1, apps/v1beta1, and extensions/v1beta1, and "rules" only included
        `apiGroups:["apps"], apiVersions:["v1"], resources: ["deployments"]`, a
        request to apps/v1beta1 or extensions/v1beta1 would be converted to apps/v1
        and sent to the webhook.
        Defaults to "Equivalent"
      * **namespaceSelector** ``Optional[meta_v1.LabelSelector]`` - NamespaceSelector decides whether to run the webhook on an object based on
        whether the namespace for that object matches the selector. If the object
        itself is a namespace, the matching is performed on object.metadata.labels. If
        the object is another cluster scoped resource, it never skips the webhook.
        For example, to run the webhook on any objects whose namespace is not
        associated with "runlevel" of "0" or "1";  you will set the selector as
        follows: "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "runlevel",
              "operator": "NotIn",
              "values": [
                "0",
                "1"
              ]
            }
          ]
        }
        If instead you want to only run the webhook on any objects whose namespace is
        associated with the "environment" of "prod" or "staging"; you will set the
        selector as follows: "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "environment",
              "operator": "In",
              "values": [
                "prod",
                "staging"
              ]
            }
          ]
        }
        See https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/
        for more examples of label selectors.
        Default to the empty LabelSelector, which matches everything.
      * **objectSelector** ``Optional[meta_v1.LabelSelector]`` - ObjectSelector decides whether to run the webhook based on if the object has
        matching labels. objectSelector is evaluated against both the oldObject and
        newObject that would be sent to the webhook, and is considered to match if
        either object matches the selector. A null object (oldObject in the case of
        create, or newObject in the case of delete) or an object that cannot have
        labels (like a DeploymentRollback or a PodProxyOptions object) is not
        considered to match. Use the object selector only if the webhook is opt-in,
        because end users may skip the admission webhook by setting the labels.
        Default to the empty LabelSelector, which matches everything.
      * **reinvocationPolicy** ``Optional[str]`` - reinvocationPolicy indicates whether this webhook should be called multiple
        times as part of a single admission evaluation. Allowed values are "Never" and
        "IfNeeded".
        Never: the webhook will not be called more than once in a single admission
        evaluation.
        IfNeeded: the webhook will be called at least one additional time as part of
        the admission evaluation if the object being admitted is modified by other
        admission plugins after the initial webhook call. Webhooks that specify this
        option *must* be idempotent, able to process objects they previously admitted.
        Note: * the number of additional invocations is not guaranteed to be exactly
        one. * if additional invocations result in further modifications to the
        object, webhooks are not guaranteed to be invoked again. * webhooks that use
        this option may be reordered to minimize the number of additional invocations.
        * to validate an object after all mutations are guaranteed complete, use a
        validating admission webhook instead.
        Defaults to "Never".
      * **rules** ``Optional[List[RuleWithOperations]]`` - Rules describes what operations on what resources/subresources the webhook
        cares about. The webhook cares about an operation if it matches _any_ Rule.
        However, in order to prevent ValidatingAdmissionWebhooks and
        MutatingAdmissionWebhooks from putting the cluster in a state which cannot be
        recovered from without completely disabling the plugin,
        ValidatingAdmissionWebhooks and MutatingAdmissionWebhooks are never called on
        admission requests for ValidatingWebhookConfiguration and
        MutatingWebhookConfiguration objects.
      * **timeoutSeconds** ``Optional[int]`` - TimeoutSeconds specifies the timeout for this webhook. After the timeout
        passes, the webhook call will be ignored or the API call will fail based on
        the failure policy. The timeout value must be between 1 and 30 seconds.
        Default to 10 seconds.
    """
    admissionReviewVersions: 'List[str]'
    clientConfig: 'WebhookClientConfig'
    name: 'str'
    sideEffects: 'str'
    failurePolicy: 'Optional[str]' = None
    matchPolicy: 'Optional[str]' = None
    namespaceSelector: 'Optional[meta_v1.LabelSelector]' = None
    objectSelector: 'Optional[meta_v1.LabelSelector]' = None
    reinvocationPolicy: 'Optional[str]' = None
    rules: 'Optional[List[RuleWithOperations]]' = None
    timeoutSeconds: 'Optional[int]' = None


@dataclass
class MutatingWebhookConfiguration(DictMixin):
    """MutatingWebhookConfiguration describes the configuration of and admission
      webhook that accept or reject and may change the object.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata; More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata.
      * **webhooks** ``Optional[List[MutatingWebhook]]`` - Webhooks is a list of webhooks and the affected resources and operations.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    webhooks: 'Optional[List[MutatingWebhook]]' = None


@dataclass
class MutatingWebhookConfigurationList(DictMixin):
    """MutatingWebhookConfigurationList is a list of MutatingWebhookConfiguration.

      **parameters**

      * **items** ``List[MutatingWebhookConfiguration]`` - List of MutatingWebhookConfiguration.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    items: 'List[MutatingWebhookConfiguration]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class RuleWithOperations(DictMixin):
    """RuleWithOperations is a tuple of Operations and Resources. It is recommended
      to make sure that all the tuple expansions are valid.

      **parameters**

      * **apiGroups** ``Optional[List[str]]`` - APIGroups is the API groups the resources belong to. '*' is all groups. If '*'
        is present, the length of the slice must be one. Required.
      * **apiVersions** ``Optional[List[str]]`` - APIVersions is the API versions the resources belong to. '*' is all versions.
        If '*' is present, the length of the slice must be one. Required.
      * **operations** ``Optional[List[str]]`` - Operations is the operations the admission hook cares about - CREATE, UPDATE,
        DELETE, CONNECT or * for all of those operations and any future admission
        operations that are added. If '*' is present, the length of the slice must be
        one. Required.
      * **resources** ``Optional[List[str]]`` - Resources is a list of resources this rule applies to.
        For example: 'pods' means pods. 'pods/log' means the log subresource of pods.
        '*' means all resources, but not subresources. 'pods/*' means all subresources
        of pods. '*/scale' means all scale subresources. '*/*' means all resources and
        their subresources.
        If wildcard is present, the validation rule will ensure resources do not
        overlap with each other.
        Depending on the enclosing object, subresources might not be allowed.
        Required.
      * **scope** ``Optional[str]`` - scope specifies the scope of this rule. Valid values are "Cluster",
        "Namespaced", and "*" "Cluster" means that only cluster-scoped resources will
        match this rule. Namespace API objects are cluster-scoped. "Namespaced" means
        that only namespaced resources will match this rule. "*" means that there are
        no scope restrictions. Subresources match the scope of their parent resource.
        Default is "*".
    """
    apiGroups: 'Optional[List[str]]' = None
    apiVersions: 'Optional[List[str]]' = None
    operations: 'Optional[List[str]]' = None
    resources: 'Optional[List[str]]' = None
    scope: 'Optional[str]' = None


@dataclass
class ServiceReference(DictMixin):
    """ServiceReference holds a reference to Service.legacy.k8s.io

      **parameters**

      * **name** ``str`` - `name` is the name of the service. Required
      * **namespace** ``str`` - `namespace` is the namespace of the service. Required
      * **path** ``Optional[str]`` - `path` is an optional URL path which will be sent in any request to this
        service.
      * **port** ``Optional[int]`` - If specified, the port on the service that hosting webhook. Default to 443 for
        backward compatibility. `port` should be a valid port number (1-65535,
        inclusive).
    """
    name: 'str'
    namespace: 'str'
    path: 'Optional[str]' = None
    port: 'Optional[int]' = None


@dataclass
class ValidatingWebhook(DictMixin):
    """ValidatingWebhook describes an admission webhook and the resources and
      operations it applies to.

      **parameters**

      * **admissionReviewVersions** ``List[str]`` - AdmissionReviewVersions is an ordered list of preferred `AdmissionReview`
        versions the Webhook expects. API server will try to use first version in the
        list which it supports. If none of the versions specified in this list
        supported by API server, validation will fail for this object. If a persisted
        webhook configuration specifies allowed versions and does not include any
        versions known to the API Server, calls to the webhook will fail and be
        subject to the failure policy.
      * **clientConfig** ``WebhookClientConfig`` - ClientConfig defines how to communicate with the hook. Required
      * **name** ``str`` - The name of the admission webhook. Name should be fully qualified, e.g.,
        imagepolicy.kubernetes.io, where "imagepolicy" is the name of the webhook, and
        kubernetes.io is the name of the organization. Required.
      * **sideEffects** ``str`` - SideEffects states whether this webhook has side effects. Acceptable values
        are: None, NoneOnDryRun (webhooks created via v1beta1 may also specify Some or
        Unknown). Webhooks with side effects MUST implement a reconciliation system,
        since a request may be rejected by a future step in the admission chain and
        the side effects therefore need to be undone. Requests with the dryRun
        attribute will be auto-rejected if they match a webhook with sideEffects ==
        Unknown or Some.
      * **failurePolicy** ``Optional[str]`` - FailurePolicy defines how unrecognized errors from the admission endpoint are
        handled - allowed values are Ignore or Fail. Defaults to Fail.
      * **matchPolicy** ``Optional[str]`` - matchPolicy defines how the "rules" list is used to match incoming requests.
        Allowed values are "Exact" or "Equivalent".
        - Exact: match a request only if it exactly matches a specified rule. For
        example, if deployments can be modified via apps/v1, apps/v1beta1, and
        extensions/v1beta1, but "rules" only included `apiGroups:["apps"],
        apiVersions:["v1"], resources: ["deployments"]`, a request to apps/v1beta1 or
        extensions/v1beta1 would not be sent to the webhook.
        - Equivalent: match a request if modifies a resource listed in rules, even via
        another API group or version. For example, if deployments can be modified via
        apps/v1, apps/v1beta1, and extensions/v1beta1, and "rules" only included
        `apiGroups:["apps"], apiVersions:["v1"], resources: ["deployments"]`, a
        request to apps/v1beta1 or extensions/v1beta1 would be converted to apps/v1
        and sent to the webhook.
        Defaults to "Equivalent"
      * **namespaceSelector** ``Optional[meta_v1.LabelSelector]`` - NamespaceSelector decides whether to run the webhook on an object based on
        whether the namespace for that object matches the selector. If the object
        itself is a namespace, the matching is performed on object.metadata.labels. If
        the object is another cluster scoped resource, it never skips the webhook.
        For example, to run the webhook on any objects whose namespace is not
        associated with "runlevel" of "0" or "1";  you will set the selector as
        follows: "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "runlevel",
              "operator": "NotIn",
              "values": [
                "0",
                "1"
              ]
            }
          ]
        }
        If instead you want to only run the webhook on any objects whose namespace is
        associated with the "environment" of "prod" or "staging"; you will set the
        selector as follows: "namespaceSelector": {
          "matchExpressions": [
            {
              "key": "environment",
              "operator": "In",
              "values": [
                "prod",
                "staging"
              ]
            }
          ]
        }
        See https://kubernetes.io/docs/concepts/overview/working-with-objects/labels
        for more examples of label selectors.
        Default to the empty LabelSelector, which matches everything.
      * **objectSelector** ``Optional[meta_v1.LabelSelector]`` - ObjectSelector decides whether to run the webhook based on if the object has
        matching labels. objectSelector is evaluated against both the oldObject and
        newObject that would be sent to the webhook, and is considered to match if
        either object matches the selector. A null object (oldObject in the case of
        create, or newObject in the case of delete) or an object that cannot have
        labels (like a DeploymentRollback or a PodProxyOptions object) is not
        considered to match. Use the object selector only if the webhook is opt-in,
        because end users may skip the admission webhook by setting the labels.
        Default to the empty LabelSelector, which matches everything.
      * **rules** ``Optional[List[RuleWithOperations]]`` - Rules describes what operations on what resources/subresources the webhook
        cares about. The webhook cares about an operation if it matches _any_ Rule.
        However, in order to prevent ValidatingAdmissionWebhooks and
        MutatingAdmissionWebhooks from putting the cluster in a state which cannot be
        recovered from without completely disabling the plugin,
        ValidatingAdmissionWebhooks and MutatingAdmissionWebhooks are never called on
        admission requests for ValidatingWebhookConfiguration and
        MutatingWebhookConfiguration objects.
      * **timeoutSeconds** ``Optional[int]`` - TimeoutSeconds specifies the timeout for this webhook. After the timeout
        passes, the webhook call will be ignored or the API call will fail based on
        the failure policy. The timeout value must be between 1 and 30 seconds.
        Default to 10 seconds.
    """
    admissionReviewVersions: 'List[str]'
    clientConfig: 'WebhookClientConfig'
    name: 'str'
    sideEffects: 'str'
    failurePolicy: 'Optional[str]' = None
    matchPolicy: 'Optional[str]' = None
    namespaceSelector: 'Optional[meta_v1.LabelSelector]' = None
    objectSelector: 'Optional[meta_v1.LabelSelector]' = None
    rules: 'Optional[List[RuleWithOperations]]' = None
    timeoutSeconds: 'Optional[int]' = None


@dataclass
class ValidatingWebhookConfiguration(DictMixin):
    """ValidatingWebhookConfiguration describes the configuration of and admission
      webhook that accept or reject and object without changing it.

      **parameters**

      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ObjectMeta]`` - Standard object metadata; More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata.
      * **webhooks** ``Optional[List[ValidatingWebhook]]`` - Webhooks is a list of webhooks and the affected resources and operations.
    """
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ObjectMeta]' = None
    webhooks: 'Optional[List[ValidatingWebhook]]' = None


@dataclass
class ValidatingWebhookConfigurationList(DictMixin):
    """ValidatingWebhookConfigurationList is a list of
      ValidatingWebhookConfiguration.

      **parameters**

      * **items** ``List[ValidatingWebhookConfiguration]`` - List of ValidatingWebhookConfiguration.
      * **apiVersion** ``Optional[str]`` - APIVersion defines the versioned schema of this representation of an object.
        Servers should convert recognized schemas to the latest internal value, and
        may reject unrecognized values. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
      * **kind** ``Optional[str]`` - Kind is a string value representing the REST resource this object represents.
        Servers may infer this from the endpoint the client submits requests to.
        Cannot be updated. In CamelCase. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
      * **metadata** ``Optional[meta_v1.ListMeta]`` - Standard list metadata. More info:
        https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    items: 'List[ValidatingWebhookConfiguration]'
    apiVersion: 'Optional[str]' = None
    kind: 'Optional[str]' = None
    metadata: 'Optional[meta_v1.ListMeta]' = None


@dataclass
class WebhookClientConfig(DictMixin):
    """WebhookClientConfig contains the information to make a TLS connection with the
      webhook

      **parameters**

      * **caBundle** ``Optional[str]`` - `caBundle` is a PEM encoded CA bundle which will be used to validate the
        webhook's server certificate. If unspecified, system trust roots on the
        apiserver are used.
      * **service** ``Optional[ServiceReference]`` - `service` is a reference to the service for this webhook. Either `service` or
        `url` must be specified.
        If the webhook is running within the cluster, then you should use `service`.
      * **url** ``Optional[str]`` - `url` gives the location of the webhook, in standard URL form
        (`scheme://host:port/path`). Exactly one of `url` or `service` must be
        specified.
        The `host` should not refer to a service running in the cluster; use the
        `service` field instead. The host might be resolved via external DNS in some
        apiservers (e.g., `kube-apiserver` cannot resolve in-cluster DNS as that would
        be a layering violation). `host` may also be an IP address.
        Please note that using `localhost` or `127.0.0.1` as a `host` is risky unless
        you take great care to run this webhook on all hosts which run an apiserver
        which might need to make calls to this webhook. Such installs are likely to be
        non-portable, i.e., not easy to turn up in a new cluster.
        The scheme must be "https"; the URL must begin with "https://".
        A path is optional, and if present may be any string permissible in a URL. You
        may use the path to pass an arbitrary string to the webhook, for example, a
        cluster identifier.
        Attempting to use a user or basic auth e.g. "user:password@" is not allowed.
        Fragments ("#...") and query parameters ("?...") are not allowed, either.
    """
    caBundle: 'Optional[str]' = None
    service: 'Optional[ServiceReference]' = None
    url: 'Optional[str]' = None


